import parallelRunAndWrite as prw
from time import sleep, time
import os
from random import randint
import re

rangeSize = 10
outFname = "output.txt"
errFname = "err.txt"
errRegex=r'prw arg \d+ pid \d+'


def raiseOrWork(arg):
    if(randint(0,100)%2 ==0):
        raise Exception("prw arg {} pid {} ".format(arg, os.getpid()))
    return work(arg)

def work(arg):
    ret=str(arg)+str(os.getpid())
    sleep(randint(0,2))
    return ret

def verifyOutput(exact=False):
    ok=True
    numLines=0
    with open(outFname, 'r') as f:
        for line in f:
            if(re.match(r'\d*',line) is None) :
                ok=False
            numLines+=1
    if(exact):
        ok = ok and numLines == rangeSize
    else:
        ok = ok and numLines >= 1 and numLines <= rangeSize
    assert ok

def verifyErrFile():
    found=False
    with open(errFname, 'r') as f:
        for line in f:
            if(re.match('Exception: '+errRegex,line) is None) :
                found=True
    assert found

def verifyFiles(err=False, exact=False):
    verifyOutput(exact)
    os.unlink(outFname)
    if(err):
        verifyErrFile()
        os.unlink(errFname)

def testErrFileDebug():
    prw.parallelRunAndWrite(raiseOrWork, range(rangeSize), outFname,  errFileName=errFname, debug=True)
    verifyFiles(err=True, exact=False)

def testErrFile():
    prw.parallelRunAndWrite(raiseOrWork, range(rangeSize), outFname,  errFileName=errFname)
    verifyFiles(err=True, exact=False)

def testNoErrFile():
    prw.parallelRunAndWrite(work, range(rangeSize), outFname)
    verifyFiles(err=False,exact=True)

def testRaise():
    from pytest import raises
    with raises(Exception, match=errRegex):
        prw.parallelRunAndWrite(raiseOrWork, range(rangeSize), outFname)
    verifyFiles(err=False,exact=False)
